package it.isti.cnr.testlab.helloworldalgorithm;

import java.io.File;
import java.io.FileWriter;
import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;

import org.gcube.contentmanagement.lexicalmatcher.utils.AnalysisLogger;
import org.gcube.dataanalysis.ecoengine.configuration.AlgorithmConfiguration;
import org.gcube.dataanalysis.ecoengine.datatypes.DatabaseType;
import org.gcube.dataanalysis.ecoengine.datatypes.PrimitiveType;
import org.gcube.dataanalysis.ecoengine.datatypes.StatisticalType;
import org.gcube.dataanalysis.ecoengine.datatypes.enumtypes.PrimitiveTypes;
import org.gcube.dataanalysis.ecoengine.datatypes.enumtypes.TableTemplates;
import org.gcube.dataanalysis.ecoengine.interfaces.StandardLocalExternalAlgorithm;

public class HelloWorldAlgorithm extends StandardLocalExternalAlgorithm {

	// Input String Names
	private String input1 = "firstInputString";
	private String input2 = "secondInputString";
	
	// Output File Placeholder
	private File outputFile = null;
	
	// Output File Name
	private String outputFileName = "output_file";
			
	@Override
	public void init() throws Exception {
		AnalysisLogger.setLogger(config.getConfigPath() + AlgorithmConfiguration.defaultLoggerFile);
		AnalysisLogger.getLogger().debug("Process Initialized");			
	}

	@Override
	public String getDescription() {
		String description = "Simple example procedure that takes two strings as input and produces a file containing the concatenation of the two input strings as output";
		return description;
	}
	
	@Override
	protected void setInputParameters() {
		List<TableTemplates> tableTemplates = new ArrayList<TableTemplates>();
		tableTemplates.add(TableTemplates.GENERIC);
		inputs.add(new PrimitiveType(String.class.getName(), null, PrimitiveTypes.STRING, input1, "First input string", "Hello"));
		inputs.add(new PrimitiveType(String.class.getName(), null, PrimitiveTypes.STRING, input2, "Second input string", "World"));
		DatabaseType.addDefaultDBPars(inputs);		
	}
	
	@Override
	protected void process() throws Exception {
		AnalysisLogger.getLogger().debug("Concatenating the two input strings");
		String outputString = getInputParameter(input1) + getInputParameter(input2);
		AnalysisLogger.getLogger().debug("--> DONE");
		try {
			AnalysisLogger.getLogger().debug("Writing output string to file");
			outputFile = new File(outputFileName);
			FileWriter fileWriter = new FileWriter(outputFile);
			fileWriter.write(outputString);
			fileWriter.close();
			AnalysisLogger.getLogger().debug("--> DONE");
		} catch (Exception e) {
			AnalysisLogger.getLogger().debug("--> ERROR");
			AnalysisLogger.getLogger().error(e.getMessage());
			throw e;
		}
	}

	@Override
	public StatisticalType getOutput() {
		LinkedHashMap<String, StatisticalType> outputHashMap = new LinkedHashMap<String, StatisticalType>();
		outputHashMap.put("Trained Neural Network", new PrimitiveType(File.class.getName(), new File(outputFile.getAbsolutePath()), PrimitiveTypes.FILE, "outputFile", "File containing the concatenation of the 2 input strings"));
		PrimitiveType output = new PrimitiveType(LinkedHashMap.class.getName(), outputHashMap, PrimitiveTypes.MAP, "resultsHashMap", "Results Hash Map");
		return output;	
	}
	
	@Override
	public void shutdown() {
		AnalysisLogger.getLogger().debug("Process Terminated");								
	}
    
}
